/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2014 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var CreateChildBioPlate = function()
{
	var child = {};
	
	// For drawing
	var graphics;
	var pen;
	var selectedPen;
	
	// Source and destination plates
	var sourcePlate;
	var destPlates;
	
	// We can select one well on each plate
	var selectedSourceWell;
	var selectedDestWell;

	// Plate and source well currently being edited
	var editingPlate;
	var editingWell;

	/**
		Initialize the page.
	*/
	child.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'step-1')
		{
			// Save and close buttons
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnNext', child.gotoStep2);
			
			// Hardware
			Buttons.addClickHandler('hardware_id.select', child.selectHardware);
			Events.addEventHandler('hardware_id', 'base-selected', Items.onItemSelected);

			// Protocol
			Buttons.addClickHandler('protocol_id.select', child.selectProtocol);
			Events.addEventHandler('protocol_id', 'base-selected', Items.onItemSelected);

			// Protocol
			Buttons.addClickHandler('kit_id.select', child.selectKit);
			Events.addEventHandler('kit_id', 'base-selected', Items.onItemSelected);

			// Tag
			Buttons.addClickHandler('tag_id.select', child.selectTag);
			Events.addEventHandler('tag_id', 'base-selected', Items.onItemSelected);
			
			// Quantities
			Events.addEventHandler('original_quantity', 'keypress', Events.numberOnly);
			Events.addEventHandler('used_quantity', 'keypress', Events.numberOnly);
			
			// Child type
			Events.addEventHandler('child_biomaterial_type', 'change', child.childTypeOnChange);

			// Plate geometry
			Buttons.addClickHandler('plategeometry_id.select', child.selectPlateGeometry);
			Events.addEventHandler('plategeometry_id', 'base-selected', Items.onItemSelected);

			// Bioplate type
			Buttons.addClickHandler('bioplatetype_id.select', child.selectBioPlateType);
			Events.addEventHandler('bioplatetype_id', 'base-selected', Items.onItemSelected);
			
			// Storage location
			Buttons.addClickHandler('freezer_id.select', child.selectFreezer);
			Events.addEventHandler('freezer_id', 'base-selected', Items.onItemSelected);
			
			// Plates
			Events.addEventHandler('number_of_plates', 'keypress', Events.integerOnly);
			Events.addEventHandler('size', 'keypress', Events.integerOnly);
			
			child.childTypeOnChange();
		}
		else if (pageId == 'step-2')
		{
			// Save and close buttons
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnSave', child.createChildPlate);
			
			// Toolbar
			Buttons.addClickHandler('btnPlaceByRow', child.placeByRow);
			Buttons.addClickHandler('btnPlaceByColumn', child.placeByColumn);
			Buttons.addClickHandler('btnClearMapping', child.clearMapping);
			Buttons.addClickHandler('btnSelectPlateMapping', child.selectPlateMapping);
			Events.addEventHandler('btnSelectPlateMapping', 'base-selected', child.setPlateMappingCallback);

			// Options
			Events.addEventHandler('showSourceCoordinates', 'click', child.showSourceCoordinatesOnClick);

			// Handle layout changes
			Events.addEventHandler(window, 'resize', child.onLayoutChange);
			
			// Initialize graphics
			graphics = new jsGraphics(Doc.element('canvas'));
			pen = new jsPen(new jsColor('#2288AA'), 1);
			selectedPen = new jsPen(new jsColor('#2288AA'), 2);

			child.initSourceBioPlate();
			child.initDestinationPlates();
		}

	}
	
	child.gotoStep2 = function()
	{
		var frm = document.forms['main'];
		var childType = frm.child_biomaterial_type.value;
		if (Strings.trim(frm.event_name.value) == '')
		{
			Forms.showNotification(frm.event_name, 'You must enter a name for the new event');
			return;
		}
		if (Strings.trim(frm.number_of_plates.value) == '')
		{
			Forms.showNotification(frm.number_of_plates, 'You must enter the number of plates that was created');
			return;
		}
		if (Strings.trim(frm.plate_name_prefix.value) == '')
		{
			Forms.showNotification(frm.plate_name_prefix, 'You must enter a name prefix for the new plates');
			return;
		}
		if (frm.plategeometry_id.length == 0 && childType != 'PHYSICALBIOASSAY')
		{
			Forms.showNotification('plategeometry_id.select', 'You must select a plate geometry');
			return false;
		}
		if (frm.bioplatetype_id.length == 0 && childType != 'PHYSICALBIOASSAY')
		{
			Forms.showNotification('bioplatetype_id.select', 'You must select a plate type');
			return false;
		}
		frm.submit();
	}
	
	child.selectHardware = function()
	{
		var frm = document.forms['main'];
		var url = '&resetTemporary=1';
		if (frm.hardware_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.hardware_id[1].value));				
			url += '&item_id='+id;
		}
		var childType = frm.child_biomaterial_type.value;
		url += ItemSubtype.createRelatedFilter(childType+'_subtype_id', 'HARDWARE');
		Dialogs.selectItem('HARDWARE', 'hardware_id', 0, url);
	}

	child.selectProtocol = function()
	{
		var frm = document.forms['main'];
		var url = '&resetTemporary=1';
		if (frm.protocol_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.protocol_id[1].value));				
			url += '&item_id='+id;
		}
		var childType = frm.child_biomaterial_type[frm.child_biomaterial_type.selectedIndex].value;
		url += ItemSubtype.createRelatedFilter(childType+'_subtype_id', 'PROTOCOL');
		Dialogs.selectItem('PROTOCOL', 'protocol_id', 0, url);
	}
	
	child.selectKit = function()
	{
		var frm = document.forms['main'];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:BOOLEAN:inactive=0';
		if (frm.kit_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.kit_id[1].value));				
			url += '&item_id='+id;
		}
		var childType = frm.child_biomaterial_type[frm.child_biomaterial_type.selectedIndex].value;
		url += ItemSubtype.createRelatedFilter(childType+'_subtype_id', 'KIT');
		Dialogs.selectItem('KIT', 'kit_id', 0, url);
	}

	child.selectTag = function()
	{
		var frm = document.forms['main'];
		var url = '&resetTemporary=1';
		if (frm.protocol_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.protocol_id[1].value));				
			url += '&item_id='+id;
		}
		var childType = frm.child_biomaterial_type.value;
		url += ItemSubtype.createRelatedFilter(childType+'_subtype_id', 'TAG');
		Dialogs.selectItem('TAG', 'tag_id', 0, url);
	}

	child.selectFreezer = function()
	{
		var frm = document.forms['main'];
		var url = '&resetTemporary=1';
		if (frm.freezer_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.freezer_id[1].value));				
			url += '&item_id='+id;
		}
		url += '&tmpfilter:INT:itemSubtype='+Data.int('page-data', 'freezer-id');
		Dialogs.selectItem('HARDWARE', 'freezer_id', 0, url);
	}
	
	child.selectPlateGeometry = function()
	{
		var frm = document.forms['main'];
		var url = '&resetTemporary=1';
		if (frm.plategeometry_id.length > 0) url += '&item_id='+frm.plategeometry_id[0].value;
		Dialogs.selectItem('PLATEGEOMETRY', 'plategeometry_id', 0, url);
	}
	
	child.selectBioPlateType = function()
	{
		var frm = document.forms['main'];
		var url = '&resetTemporary=1';
		if (frm.bioplatetype_id.length > 0)
		{
			url += '&item_id='+frm.bioplatetype_id[0].value;
		}
		
		var childType = frm.child_biomaterial_type.value; 
		// Restrict to plates that can holds to target biomaterial
		url += '&tmpfilter:INT:bioMaterialType='+encodeURIComponent('|'+Data.int('page-data', childType));
		var subtypeId = ItemSubtype.getSubtypeId(childType+'_subtype_id');
		// Restrict to plates with the given subtype
		url += '&tmpfilter:INT:itemSubtype='+encodeURIComponent(subtypeId ? '|' + subtypeId : '=');
	
		Dialogs.selectItem('BIOPLATETYPE', 'bioplatetype_id', 0, url);
	}
	
	child.childTypeOnChange = function()
	{
		var frm = document.forms['main'];
		var childType = frm.child_biomaterial_type.value;
		var isSample = childType == 'SAMPLE';
		var isExtract = childType == 'EXTRACT';
		var isBioAssay = childType == 'PHYSICALBIOASSAY';
		Doc.showHide('sampleSubtypesDiv', isSample);
		Doc.showHide('extractSubtypesDiv', isExtract);
		Doc.showHide('bioAssaySubtypesDiv', isBioAssay);
		Doc.showHide('tagDiv', isExtract);
		Doc.showHide('originalQuantityDiv', !isBioAssay);
		Doc.showHide('freezerDiv', !isBioAssay);
		Doc.showHide('plateTypeDiv', !isBioAssay);
		Doc.showHide('geometryDiv', !isBioAssay);
		Doc.showHide('sizeDiv', isBioAssay);
	}

	
	/**
		Select the source well that is clicked on. The currently selected well
		is de-selected. If the clicked well is the same as the currently 
		selected well no new item is selected. If a destination well is already 
		selected a link is made between the source and destination wells.
	*/
	child.sourceWellOnClick = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var column = Data.int(event.currentTarget, 'column');
		
		var well = sourcePlate.getWell(row, column);
		if (!well) return;
		child.storeAndHideChildInfo();
	
		// De-select the currently selected source well
		if (selectedSourceWell) 
		{
			selectedSourceWell.setSelected(false);
			if (well == selectedSourceWell)
			{
				// Re-draw link with regular pen (eg. same as onmouseover)
				selectedSourceWell.drawLink(graphics, pen, true);
				selectedSourceWell = null;
				return;
			}
			else
			{
				selectedSourceWell.hideLink(graphics);
			}
		}
		
		// Select the new source well and draw link to destination well
		selectedSourceWell = well;
		selectedSourceWell.setSelected(true);
		selectedSourceWell.drawLink(graphics, selectedPen, true);
	
		// Map the source and destination wells
		if (selectedSourceWell && selectedDestWell)
		{
			child.mapSelectedWells();
		}
		child.showChildInfo();
	}
	
	/**
		Draw a link between the mapped source and destination wells.
	*/
	child.sourceWellOnMouseOver = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var column = Data.int(event.currentTarget, 'column');

		var well = sourcePlate.getWell(row, column);
		if (!well) return;
		well.drawLink(graphics, pen, false);
	}
	
	/**
		Hide the link between the mapped source and destination 
		wells, unless one of them is selected.
	*/
	child.sourceWellOnMouseOut = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var column = Data.int(event.currentTarget, 'column');

		var well = sourcePlate.getWell(row, column);
		if (!well) return;
		if (!well.selected && !(well.mappedWell && well.mappedWell.selected)) 
		{
			well.hideLink(graphics);
		}
	}
	
	child.destWellOnClick = function(event)
	{
		var plateNo = Data.int(event.currentTarget, 'plate');
		var row = Data.int(event.currentTarget, 'row');
		var column = Data.int(event.currentTarget, 'column');
		
		var well = destPlates[plateNo].getWell(row, column);
		child.storeAndHideChildInfo();
	
		// De-select the currently selected well
		if (selectedDestWell) 
		{
			selectedDestWell.setSelected(false);
			if (well == selectedDestWell)
			{
				// Re-draw link with regular pen (eg. same as onmouseover)
				selectedDestWell.drawLink(graphics, pen, true);
				selectedDestWell = null;
				return;
			}
			else
			{
				selectedDestWell.hideLink(graphics);
			}
		}
		
		selectedDestWell = well;
		selectedDestWell.setSelected(true);
		selectedDestWell.drawLink(graphics, selectedPen, true);
	
		if (selectedSourceWell && selectedDestWell)
		{
			child.mapSelectedWells();
		}
		child.showChildInfo();
	}
	
	child.destWellOnMouseOver = function(event)
	{
		var plateNo = Data.int(event.currentTarget, 'plate');
		var row = Data.int(event.currentTarget, 'row');
		var column = Data.int(event.currentTarget, 'column');
		
		var well = destPlates[plateNo].getWell(row, column);
		if (!well) return;
		well.drawLink(graphics, pen, false);
	}
	
	child.destWellOnMouseOut = function(event)
	{
		var plateNo = Data.int(event.currentTarget, 'plate');
		var row = Data.int(event.currentTarget, 'row');
		var column = Data.int(event.currentTarget, 'column');
		
		var well = destPlates[plateNo].getWell(row, column);
		if (!well) return;
		if (!well.selected && !(well.mappedWell && well.mappedWell.selected)) 
		{
			well.hideLink(graphics);
		}
	}
	
	/**
		Map the selected source and destination wells. Hide and
		redraw links as needed.
	*/
	child.mapSelectedWells = function()
	{
		// Hide any links that are currently displayed
		selectedSourceWell.hideLink(graphics);
		selectedDestWell.hideLink(graphics);
		
		// Map to the new well and draw link
		if (selectedSourceWell.mappedWell != selectedDestWell)
		{
			selectedSourceWell.mapToWell(selectedDestWell);
			selectedSourceWell.drawLink(graphics, pen, true);
		}
		else
		{
			selectedSourceWell.unmapWell();
		}
		
		// De-select everything
		selectedSourceWell.setSelected(false);
		selectedDestWell.setSelected(false);
		selectedSourceWell = null;
		selectedDestWell = null;
	}

	/**
		Remove all mappings that have been made so far.
	*/
	child.clearMapping = function()
	{
		if (!destPlates) 
		{
			alert('No destination plates have been created');
			return;
		}
		if (!confirm('This will remove all placed biomaterial. Continue?')) return;
		sourcePlate.unmapAll(graphics);
	}
	
	/**
		Automatically move remaining biomaterial to the destination plate filling rows first.
		Biomaterial and wells that have already been mapped are skipped.
	*/
	child.placeByRow = function()
	{
		if (!destPlates) 
		{
			alert('No destination plates have been created');
			return;
		}
		var srcRow = 0;
		var srcCol = 0;
		for (var plateNo = 0; plateNo < destPlates.length; plateNo++)
		{
			var destPlate = destPlates[plateNo];
			for (var destRow = 0; destRow < destPlate.rows; destRow++)
			{
				for (var destCol = 0; destCol < destPlate.columns; destCol++)
				{
					var destWell = destPlate.getWell(destRow, destCol);
					if (!destWell.mappedWell && !destWell.locked)
					{
						var mapped = false;
						var srcWell = sourcePlate.getWell(srcRow, srcCol);
						while (srcWell && !mapped)
						{
							if (srcWell.id && !srcWell.locked && !srcWell.mappedWell)
							{
								srcWell.mapToWell(destWell);
								mapped = true;
							}
							else
							{
								srcCol++;
								if (srcCol >= sourcePlate.columns)
								{
									srcCol = 0;
									srcRow++;
								}
								srcWell = sourcePlate.getWell(srcRow, srcCol);
							}
						}
					}
				}
			}
		}
	}

	/**
		Automatically move remaining biomaterial to the destination plate filling columns first.
		Biomaterial and wells that have already been mapped are skipped.
	*/
	child.placeByColumn = function()
	{
		if (!destPlates) 
		{
			alert('No destination plates have been created');
			return;
		}
		var srcRow = 0;
		var srcCol = 0;
		for (var plateNo = 0; plateNo < destPlates.length; plateNo++)
		{
			var destPlate = destPlates[plateNo];
			for (var destCol = 0; destCol < destPlate.columns; destCol++)
			{
				for (var destRow = 0; destRow < destPlate.rows; destRow++)
				{
					var destWell = destPlate.getWell(destRow, destCol);
					if (!destWell.mappedWell && !destWell.locked)
					{
						var mapped = false;
						var srcWell = sourcePlate.getWell(srcRow, srcCol);
						while (srcWell && !mapped)
						{
							if (srcWell.id && !srcWell.locked && !srcWell.mappedWell)
							{
								srcWell.mapToWell(destWell);
								mapped = true;
							}
							else
							{
								srcRow++;
								if (srcRow >= sourcePlate.rows)
								{
									srcCol++;
									srcRow = 0;
								}
								srcWell = sourcePlate.getWell(srcRow, srcCol);
							}
						}
					}
				}
			}
		}
	}

	child.selectPlateMapping = function()
	{
		if (!destPlates) 
		{
			alert('No destination plates have been created');
			return;
		}
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:sourceGeometry.rows='+sourcePlate.rows;
		url += '&tmpfilter:INT:sourceGeometry.columns='+sourcePlate.columns;
		url += '&tmpfilter:INT:sourceCount=1';
		url += '&tmpfilter:INT:destinationGeometry.rows='+destPlates[0].rows;
		url += '&tmpfilter:INT:destinationGeometry.columns='+destPlates[0].columns;
		url += '&tmpfilter:INT:destinationCount=' + destPlates.length;
		Dialogs.selectItem('PLATEMAPPING', 'btnSelectPlateMapping', 0, url);
	}
	
	child.setPlateMappingCallback = function(event)
	{
		var request = Ajax.getXmlHttpRequest();
		var url = '../../lims/platemappings/ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=GetMappingDetails&item_id=' + event.detail.id;
		request.open("GET", url, false); 
		request.send(null);
		var mappingInfo = JSON.parse(request.responseText);
		if (mappingInfo.status != 'ok')
		{
			Forms.showNotification('btnSelectPlateMapping', plateInfo.message);
			return false;
		}
		
		var win = Dialogs.getDialog('SelectPLATEMAPPING') || window;
		if (mappingInfo.sourcePlates != 1 || mappingInfo.sourceRows != sourcePlate.rows || mappingInfo.sourceColumns != sourcePlate.columns)
		{
			if (!win.confirm("The selected mapping doesn't match the geometry of the source plate.\nContinue mapping overlapping positions?")) return;
		}
		else if (mappingInfo.destinationPlates != destPlates.length || mappingInfo.destinationRows != destPlates[0].rows || mappingInfo.destinationColumns != destPlates[0].columns)
		{
			if (!win.confirm("The selected mapping doesn't match the geometry of the destination plate.\nContinue mapping overlapping positions?")) return;
		}
		
		var numMapped = 0;
		for (var i = 0; i < mappingInfo.details.length; i++)
		{
			var mapping = mappingInfo.details[i].split(',');
			var srcWell = sourcePlate.getWell(parseInt(mapping[1]), parseInt(mapping[2]));
			var plateNo = parseInt(mapping[3]);
			if (plateNo >= 0 && plateNo < destPlates.length)
			{
				var destPlate = destPlates[plateNo];
				var destWell = destPlate.getWell(parseInt(mapping[4]), parseInt(mapping[5]));
				if (srcWell && destWell)
				{
					if (!destWell.mappedWell && !destWell.locked && srcWell.id && !srcWell.locked && !srcWell.mappedWell)
					{
						srcWell.mapToWell(destWell);
						numMapped++;
					}
				}
			}
		}
	}

	child.initSourceBioPlate = function()
	{
		var frm = document.forms['main'];
		var childBioMaterialType = frm.child_biomaterial_type.value;
		var sourceBioMaterialType = frm.source_biomaterial_type.value;
		var isBioAssayEvent = childBioMaterialType == 'PHYSICALBIOASSAY';
		var childNameSuffix = '';
		if (childBioMaterialType == 'SAMPLE')
		{
			childNameSuffix = 's';
		}
		else if (childBioMaterialType == 'EXTRACT')
		{
			childNameSuffix = 'e';
		}

		var request = Ajax.getXmlHttpRequest();
		var url = '../bioplates/ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=GetFullPlateInfo&item_id=' + frm.sourceplate_id.value;
		if (!isBioAssayEvent)
		{
			url += '&countChildren='+(sourceBioMaterialType == childBioMaterialType ? 'pooled' : '1');
		}
		request.open("GET", url, false); 
		// NOTE! 'false' causes code to wait for the response. aka. 'Synchronous AJAX' or SJAX.
		request.send(null);
		var plateInfo = JSON.parse(request.responseText);
		if (plateInfo.status != 'ok')
		{
			alert(plateInfo.message);
			return false;
		}

		// Get plate and well information from the AJAX response
		var rows = plateInfo.rows;
		var columns = plateInfo.columns;
		
		var bigPlate = rows > 12 || columns > 18;
		var plateClass = bigPlate ? 'plate bigplate' : 'plate';

		// Create plate and wells
		var prefix = 'source';
		var plate = new Plate(prefix, plateInfo.id, plateInfo.name, rows, columns, true);
		for (var i = 0; i < plateInfo.wells.length; i++)
		{
			var wellInfo = plateInfo.wells[i];
			var bmInfo = wellInfo.bioMaterial;
			var row = wellInfo.row;
			var col = wellInfo.column;
			var well = plate.getWell(row, col);
			if (bmInfo)
			{
				well.id = bmInfo.id;
				well.name = bmInfo.name;
				if (!isBioAssayEvent)
				{
					well.childCount = bmInfo.childCount;
					well.childName = well.name + '.' + childNameSuffix + (well.childCount + 1);
				}
			}
		}
		
		// Create html table representing the bioplate
		var html = '<table class="'+plateClass+'">';
		html += '<tr><td></td>';
		for (var c = 0; c < plate.columns; c++)
		{
			html += '<td class="columnheader">' + (c+1) + '</td>';
		}
		html += '</tr>';
		for (var r = 0; r < plate.rows; r++)
		{
			html += '<tr><td class="rowheader">' + Plates.toAlphaCoordinate[r] + '</td>';
			for (var c = 0; c < plate.columns; c++)
			{
				var well = plate.getWell(r, c);
				var cls = 'well';
				var onclick = '';
				var onmouseover = '';
				var onmouseout = '';
				var title = well.name ? Strings.encodeTags(well.name) : '';

				// We can only create a child biomaterial if the well is not empty
				cls += well.id ? ' used editable' : ' empty';
				html += '<td id="'+prefix+'.'+r+'.'+c+'" class="' + cls + '" data-row="'+r+'" data-column="'+c+'" title="'+title+'">';
				html += '<div class="info" id="'+prefix+'.'+r+'.'+c+'.info"></div></td>';
			}
			html += '</tr>';
		}
		html += '</table>';
		Doc.element('plate.src').innerHTML = html;
		Doc.element('plate.src.name').innerHTML = Strings.encodeTags(plateInfo.name);
		sourcePlate = plate;
		
		var wells = Doc.element('plate.src').getElementsByClassName('editable');
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			Events.addEventHandler(well, 'click', child.sourceWellOnClick);
			Events.addEventHandler(well, 'mouseout', child.sourceWellOnMouseOut);
			Events.addEventHandler(well, 'mouseover', child.sourceWellOnMouseOver);
		}
	}
	
	
	child.initDestinationPlates = function()
	{
		var frm = document.forms['main'];
		var numPlates = parseInt(frm.number_of_plates.value);
		var childBioMaterialType = frm.child_biomaterial_type.value;
		var rows = parseInt(frm.rows.value);
		var columns = parseInt(frm.columns.value);
		var namePrefix = frm.childplate_prefix.value;
		
		var bigPlate = rows > 12 || columns > 18;
		var plateClass = bigPlate ? 'plate bigplate' : 'plate';

		Doc.element('plate.dest.info').innerHTML = numPlates + ' - ' + rows + '×' + columns;
		
		// Create plate and wells
		destPlates = new Array();
		var html = '';
		for (var plateNo = 0; plateNo < numPlates; plateNo++)
		{
			var prefix = 'dest.' + plateNo;
			var plate = new Plate(prefix, plateNo, namePrefix + plateNo, rows, columns, false);
			plate.barcode = '';
			destPlates[plateNo] = plate;

			// Create html table representing the bioplate
			html += '<table class="'+plateClass+'" style="margin-bottom: 4px;">';
			if (plateNo == 0)
			{
				html += '<tr><td></td>';
				for (var c = 0; c < plate.columns; c++)
				{
					html += '<td class="columnheader">' + (c+1) + '</td>';
				}
				html += '</tr>';
			}
			for (var r = 0; r < plate.rows; r++)
			{
				html += '<tr><td class="rowheader">' + Plates.toAlphaCoordinate[r] + '</td>';
				for (var c = 0; c < plate.columns; c++)
				{
					var cls = 'well empty editable';
					html += '<td id="'+prefix+'.'+r+'.'+c+'" class="' + cls + '" data-plate="'+plateNo+'" data-row="'+r+'" data-column="'+c+'">';
					html += '<div class="info" id="'+prefix+'.'+r+'.'+c+'.info"></div></td>';
				}
				html += '</tr>';
			}
			html += '</table>';
		}
		
		Doc.element('plate.dest').innerHTML = html;

		Doc.hide('destplatecreateoptions');
		Doc.show('toolbar.mappings');
		Doc.showHide('plate.dest.options', !bigPlate);
		
		var wells = Doc.element('plate.dest').getElementsByClassName('editable');
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			Events.addEventHandler(well, 'click', child.destWellOnClick);
			Events.addEventHandler(well, 'mouseout', child.destWellOnMouseOut);
			Events.addEventHandler(well, 'mouseover', child.destWellOnMouseOver);
		}

		child.showSourceCoordinatesOnClick();
	}

	/**
		Redraw the link between selected wells when the layout changes
	*/
	child.onLayoutChange = function(event)
	{
		if (selectedSourceWell)
		{
			selectedSourceWell.drawLink(graphics, selectedPen, true);
		}
		else if (selectedDestWell)
		{
			selectedDestWell.drawLink(graphics, selectedPen, true);
		}
	}

	child.showSourceCoordinatesOnClick = function()
	{
		var frm = document.forms['main'];
		Doc.addOrRemoveClass('plate.dest', 'noinfo', !frm.showSourceCoordinates.checked);
	}

	
	child.showChildInfo = function()
	{
		if (selectedDestWell) 
		{
			editingPlate = selectedDestWell.plate;
			editingWell = selectedDestWell.mappedWell;
		}
		else if (selectedSourceWell)
		{
			if (selectedSourceWell.mappedWell)
			{
				editingPlate = selectedSourceWell.mappedWell.plate;	
			}
			editingWell = selectedSourceWell;
		}
		var frm = document.forms['main'];
		var isBioAssayEvent = frm.child_biomaterial_type.value == 'PHYSICALBIOASSAY';
		if (editingPlate)
		{
			frm.plate_name.value = editingPlate.name;
			if (!isBioAssayEvent)
			{
				frm.plate_barcode.value = editingPlate.barcode;
			}
			Doc.show('childplate.info', 'table');
		}
		if (editingWell && !isBioAssayEvent)
		{
			frm.biomaterial_name.value = editingWell.childName;
			Doc.show('childbiomaterial.info', 'table');
		}
	}
	
	child.storeAndHideChildInfo = function()
	{
		var frm = document.forms['main'];
		var isBioAssayEvent = frm.child_biomaterial_type.value == 'PHYSICALBIOASSAY';
		if (editingWell && !isBioAssayEvent)
		{
			Doc.hide('childbiomaterial.info');
			editingWell.childName = frm.biomaterial_name.value;
			editingWell = null;
		}
		if (editingPlate)
		{
			Doc.hide('childplate.info');
			editingPlate.name = frm.plate_name.value;
			if (!isBioAssayEvent)
			{
				editingPlate.barcode = frm.plate_barcode.value;
			}
			editingPlate = null;
		}
		
	}

	child.createChildPlate = function()
	{
		var frm = document.forms['main'];
		var numMapped = 0;
		var isBioAssayEvent = frm.child_biomaterial_type.value == 'PHYSICALBIOASSAY';
		for (var plateNo = 0; plateNo < destPlates.length; plateNo++)
		{
			var destPlate = destPlates[plateNo];
			Forms.addHidden(frm, 'plate.' + plateNo + '.name', destPlate.name);
			if (!isBioAssayEvent)
			{
				Forms.addHidden(frm, 'plate.' + plateNo + '.barcode', destPlate.barcode);
			}
			for (var row = 0; row < destPlate.rows; row++)
			{
				for (var column = 0; column < destPlate.columns; column++)
				{
					var well = destPlate.getWell(row, column);
					if (well.mappedWell)
					{
						var prefix = 'well.' +plateNo + '.' + row + '.' + column;
						Forms.addHidden(frm, prefix, well.mappedWell.id);
						if (!isBioAssayEvent)
						{
							Forms.addHidden(frm, prefix + '.name', well.mappedWell.childName);
						}
						numMapped++;
					}
				}
			}
		}
		
		if (numMapped == 0)
		{
			Forms.showNotification('btnSave', 'No wells have been mapped.');
			return;
		}
		
		frm.submit();
	}

	
	return child;
}();

Doc.onLoad(CreateChildBioPlate.initPage);
