/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Wizard = function()
{
	var wizard = {};
	
	/**
		Initialize the page.
	*/
	wizard.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'WizardStep1')
		{
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnNext', wizard.gotoStep2);
			
			Events.addEventHandler('quantity', 'keypress', Events.integerOnly);
			Events.addEventHandler('start_at', 'keypress', Events.integerOnly);
			Events.addEventHandler('pad_length', 'keypress', Events.integerOnly);

			// Array batch
			Buttons.addClickHandler('arraybatch_id.select', wizard.selectArrayBatch);
			Events.addEventHandler('arraybatch_id', 'base-selected', Items.onItemSelected);
			
		}
		else if (pageId == 'WizardStep2')
		{
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnSave', wizard.finishWizard);
			
			Buttons.addClickHandler('pasteMultipleName', wizard.pasteMultiple);
			Buttons.addClickHandler('clearAllNames', wizard.clearAll);
			Buttons.addClickHandler('pasteMultipleBarcode', wizard.pasteMultiple);
			Buttons.addClickHandler('clearAllBarcodes', wizard.clearAll);
			
			var frm = document.forms['WizardStep2'];
			frm.barcode0.focus();
		}
		else if (pageId == 'paste-multiple')
		{
			Buttons.addClickHandler('btnOk', wizard.saveMultiple);
			Buttons.addClickHandler('close', App.closeWindow);
			wizard.initPasteMultiple();
		}
	}
	
	wizard.gotoStep2 = function()
	{			
		if (wizard.validateStep1())
		{
			var frm = document.forms['WizardStep1'];
			frm.submit();
		}
	}		
	
	// Submit and finish the wizard
	wizard.finishWizard = function()
	{
		if (wizard.validateStep2())
		{
			var frm = document.forms['WizardStep2'];		
			frm.submit();
		}
	}

	wizard.validateStep1 = function()
	{
		var frm = document.forms['WizardStep1'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (frm.arraybatch_id && !frm.arraybatch_id.value)
		{
			Forms.showNotification('arraybatch_id.select', 'You must select an array batch');
			return false;
		}
		if (Strings.trim(frm.quantity.value) == '')
		{
			Forms.showNotification(frm.quantity, 'You must enter a quantity of slides');
			return false;
		}
		if (frm.quantity.value < 1 || frm.quantity.value > 999)
		{
			Forms.showNotification(frm.quantity, 'The quantity of array slides to be created must be between 1 and 999');
			return false;
		}
		return true;
	}

	wizard.validateStep2 = function()
	{
		var frm = document.forms['WizardStep2'];
		var quantity = parseInt(frm.quantity.value);
		for (i = 0; i < quantity; i++)
		{
			var name = frm['name'+i];
			if (Strings.trim(name.value) == '')
			{
				Forms.showNotification(name, 'You must enter a name for all slides', null, 'pointer-left');
				return false;
			}
		}
		return true;
	}

	wizard.selectArrayBatch = function()
	{
		var frm = document.forms['WizardStep1'];
		var url = '';
		if (frm.arraybatch_id.length > 0) 
		{
			url += '&item_id='+frm.arraybatch_id[0].value;
		}
		Dialogs.selectItem('ARRAYBATCH', 'arraybatch_id', 0, url);
	}
	
	wizard.pasteMultiple = function(event)
	{
		var url = 'paste_multiple.jsp?ID='+App.getSessionId();
		url += '&form=WizardStep2&prefix='+Data.get(event.currentTarget, 'form-prefix');
		Dialogs.openPopup(url, 'PasteMultiple', 400, 600);
	}

	wizard.clearAll = function(event)
	{
		var frm = document.forms['WizardStep2'];
		var i = 0;
		var prefix = Data.get(event.currentTarget, 'form-prefix');
		var field;
		while(field = frm[prefix+i])
		{
			field.value = '';
			i++;
		}
	}

	wizard.initPasteMultiple = function()
	{
		var formName = Data.get('page-data', 'form-name');
		var prefix = Data.get('page-data', 'form-prefix');
		var pfrm = window.opener.document.forms[formName];
		var frm = document.forms['paste'];
		if (pfrm)
		{
			var i = 0;
			var text = '';
			while (pfrm[prefix+i])
			{
				text += pfrm[prefix+i].value + '\n';
				i++;
			}
			frm['text'].value = text;
		}
		frm['text'].focus();
	}

	// Submit the form
	wizard.saveMultiple = function()
	{
		var formName = Data.get('page-data', 'form-name');
		var prefix = Data.get('page-data', 'form-prefix');
		var pfrm = window.opener.document.forms[formName];
		var frm = document.forms['paste'];
		if (pfrm)
		{
			var text = frm['text'].value.split('\n');
			for (var i = 0; i < text.length; ++i)
			{
				var field = prefix + i;
				if (pfrm[field]) pfrm[field].value = text[i];
			}
			// Clear any remaining fields
			while (pfrm[prefix+i])
			{
				pfrm[prefix+i].value = '';
				i++;
			}
		}
		window.close();
	}

	
	return wizard;
}();

Doc.onLoad(Wizard.initPage);
