/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Experiments = function()
{
	var experiments = {};
	
	/**
		Initialize the page.
	*/
	experiments.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', experiments.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', Annotations.onSwitchToAnnotationsTab);
			TabControl.addTabValidator('settings.info', experiments.validateExperiment);
			
			// RawBioAssays
			Buttons.addClickHandler('btnAddRawBioAssays', experiments.addRawBioAssays);
			
			// Directory
			Buttons.addClickHandler('directory_id.select', experiments.selectDirectory);
			Events.addEventHandler('directory_id', 'base-selected', Items.onItemSelected);
			
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'EXPERIMENT', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnCloneReporters', experiments.cloneReporters);
			Buttons.addClickHandler('btnRemoveClonedReporters', experiments.removeClonedReporters);
			
			TabControl.addTabActivateListener('main.bioassaysets', experiments.viewBioAssaySets);
			TabControl.addTabActivateListener('main.clonedreporters', experiments.viewClonedReporters);
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.overview', Overview.loadOnce);
			TabControl.addTabActivateListener('main.history', History.loadOnce);

			experiments.initUsedFactorValues();
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'EXPERIMENT'};
			var tableAttributes = {'table-id': 'experiments'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	experiments.initElements = function(element, autoInit)
	{
		if (autoInit == 'edit-experimental-factor')
		{
			Buttons.addClickHandler(element, experiments.editExperimentalFactor);
		}
	}
	
	experiments.viewBioAssaySets = function()
	{
		var experimentId = Data.get('page-data', 'item-id');
		var url = 'bioassaysets/index.jsp?ID='+App.getSessionId();
		url += '&experiment_id='+experimentId;
		location.replace(url);
	}
	
	experiments.viewClonedReporters = function()
	{
		var experimentId = Data.get('page-data', 'item-id');
		var url = 'reporters/index.jsp?ID='+App.getSessionId();
		url += '&experiment_id='+experimentId;
		location.replace(url);
	}
	
	experiments.cloneReporters = function()
	{
		var experimentId = Data.get('page-data', 'item-id');
		var url = 'clone_reporters.jsp?ID='+App.getSessionId();
		url += '&item_id='+experimentId;
		Dialogs.openPopup(url, 'CloneReporters', 600, 400);
	}
	
	experiments.removeClonedReporters = function()
	{
		if (confirm('Are you sure that you want to remove all cloned reporter annotations?\nThis action can\'t be undone.'))
		{
			var experimentId = Data.get('page-data', 'item-id');
			var url = 'index.jsp?ID='+App.getSessionId();
			url += '&cmd=RemoveClonedReporters&item_id='+experimentId;
			Dialogs.openPopup(url, 'RemoveClonedReporters', 450, 300);
		}
	}
	
	experiments.editExperimentalFactor = function(event)
	{
		var url = App.getRoot() + 'common/annotations/annotate.jsp?ID='+App.getSessionId();
		url += '&item_type='+Data.get(event.currentTarget, 'item-type');
		url += '&item_id='+Data.get(event.currentTarget, 'item-id');
		url += '&annotationtype_id='+Data.get(event.currentTarget, 'annotation-type');
		url += '&annotation_id='+Data.get(event.currentTarget, 'annotation');
		url += '&standalone=1';
		Dialogs.openPopup(url, 'EditAnnotation', 750, 500);
	}
	
	experiments.initUsedFactorValues = function()
	{
		var factorValues = Data.json('factor-values', 'factor-values');
		var numRawBioAssays = Data.int('factor-values', 'num-raw-bioassays');
		var numIncomplete = 0;
		for (var i = 0; i < factorValues.length; i++)
		{
			var fv = factorValues[i];
			if (fv.usedValues != undefined)
			{
				var div = Doc.element('usedvalues.'+fv.id);
				div.innerHTML = Strings.encodeTags(fv.usedValues);
			}
			
			var statusDiv = Doc.element('status.'+fv.id);
			var status = '';
			if (fv.numRawBioAssaysWithFactor >= numRawBioAssays)
			{
				status = '<img src="../../images/ok.png" alt="Ok" title="All ' + numRawBioAssays + 
					' raw bioassays have a value for this experimental factor">';
			}
			else if (fv.disableInheritance)
			{
				status = '<img src="../../images/notify.png" alt="!">&nbsp;Inheritance not allowed';
			}
			else if (fv.usePermission)
			{
				numIncomplete++;
				var numMissing = numRawBioAssays - fv.numRawBioAssaysWithFactor;
				status = '<img src="../../images/warning.png" alt="!"';
				status += ' title="' + numMissing + ' raw bioassays are missing a value for this experimental factor">';
				status += '<input type="checkbox" name="autoinherit" id="autoinherit'+fv.id+'" value="'+fv.id+'" checked';
				status += ' title="Select to automatically inherit annotations from parents"></td>';
				status += '<label for="autoinherit'+fv.id+'">' + numMissing + ' missing</label>';
			}
			else
			{
				status = 'Read-only';
			}
			statusDiv.innerHTML = status;
		}
		
		if (numIncomplete > 0)
		{
			var checkAll = '';
			checkAll += '<span id="check.uncheck" data-table-id="tbl.experimentalFactors" data-regexp="autoinherit" class="link" title="Toggle all (use CTRL, ALT or SHIFT to check/uncheck)">';
			checkAll += '<img src="../../images/check_uncheck.png" alt="x"></span>';
			checkAll += '&nbsp;<span id="autoInherit" class="link"';
			checkAll += ' title="Automatically try to inherit values for the selected experimental factors from parent items">';
			checkAll += 'Auto-inherit&hellip;</span>'
			Doc.element('tbl.experimentalFactors.status').innerHTML = checkAll;
			Doc.show('warning.missing');
			
			Events.addEventHandler('check.uncheck', 'click', Table.checkUncheckOnClick);
			Events.addEventHandler('autoInherit', 'click', experiments.autoInheritFactors);
		}
	}

	experiments.autoInheritFactors = function()
	{
		var frm = document.forms['tbl.experimentalFactors'];
		if (!Table.checkIfSelected('tbl.experimentalFactors', /autoinherit/)) return;
		Table.submitToPopup('tbl.experimentalFactors', 'AutoInherit', 750, 500);
	}

	experiments.validateExperiment = function()
	{
		var frm = document.forms['experiment'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (frm.rawdatatype)
		{
			if (frm.rawdatatype.value == '')
			{
				Forms.showNotification(frm.rawdatatype, 'You must select a raw data type');
				return false;
			}
		}
		return true;
	}
	
	experiments.save = function()
	{
		var frm = document.forms['experiment'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			Link.exportActions('rawBioAssays');
			Link.exportActions('annotationTypes');
			frm.submit();
		}
	}

	experiments.addRawBioAssays = function()
	{
		var frm = document.forms['experiment'];
		var currentRawBioAssays = Link.getIdsInList(frm.rawBioAssays, 'RAWBIOASSAY');
		var rawDataType = frm.rawdatatype ? frm.rawdatatype.value : Data.get('page-data', 'current-raw-data-type');

		var url = "&resetTemporary=1&exclude="+currentRawBioAssays.join(',');
		url += '&tmpfilter:STRING:rawDataType='+rawDataType;
	
		Dialogs.selectItem('RAWBIOASSAY', 'rawBioAssays', 1, url);
	}
	
	experiments.selectDirectory = function()
	{
		var frm = document.forms['experiment'];
		var url = '&resetTemporary=1';
		if (frm.directory_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.directory_id[1].value));				
			url += '&directory_id='+id;
		}
		Dialogs.selectItem('DIRECTORY', 'directory_id', 0, url);
	}
	
	return experiments;
}();

Doc.onLoad(Experiments.initPage);
Doc.addElementInitializer(Experiments.initElements);

