/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Jobs = function()
{
	var jobs = {};
	
	/**
		Initialize the page.
	*/
	jobs.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'JOB', 'item-id': itemId};
			
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Events.addEventHandler('btnDeletePermanently', 'base-notify', jobs.jobDeleted);
			
			// Pause/Abort button
			Buttons.addClickHandler('btnPause', jobs.pauseJob);
			Buttons.addClickHandler('btnAbort', jobs.abortJob);
			Buttons.addClickHandler('btnForceError', jobs.forceError);

			// Restart job
			Buttons.addClickHandler('btnRestartJob', jobs.restartJob);
			Buttons.addClickHandler('btnReconfigureJob', jobs.reconfigureJob);
			
			Buttons.addClickHandler('close', App.closeWindow);
			
			if (Data.int('page-data', 'auto-update'))
			{
				jobs.sendProgressUpdateRequest();
			}
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'JOB'};
			var tableAttributes = {'table-id': 'jobs'};
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnAbort', jobs.abortJobs, tableAttributes);
			Buttons.addClickHandler('btnPause', jobs.pauseJobs, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	jobs.jobDeleted = function(event)
	{
		window.opener.Items.list('JOB'); 
		window.close();
	}
	
	jobs.pauseJob = function()
	{
		var jobId = Data.get('page-data', 'item-id');
		var url = 'index.jsp?ID='+App.getSessionId();
		url += '&cmd=PauseJob&item_id='+jobId;
		location.replace(url);
	}

	jobs.pauseJobs = function()
	{
		var frm = document.forms['jobs'];
		var numSelected = Table.checkIfSelected('jobs');
		if (numSelected == 0) return;
		if (!confirm('You are about to pause '+numSelected+' jobs. Continue?'))
		{
			return;
		}
		frm.cmd.value = 'PauseJobs';
		frm.submit();
	}

	jobs.abortJobs = function()
	{
		var frm = document.forms['jobs'];
		var numSelected = Table.checkIfSelected('jobs');
		if (numSelected == 0) return;
		if (!confirm('You are about to abort '+numSelected+' jobs. This can\'t be undone. Continue?'))
		{
			return;
		}
		frm.cmd.value = 'AbortJobs';
		frm.submit();
	}
	
	jobs.abortJob = function()
	{
		if (confirm('Are you sure? This action may not be undone'))
		{
			var jobId = Data.get('page-data', 'item-id');
			var url = 'index.jsp?ID='+App.getSessionId();
			url += '&cmd=AbortJob&item_id='+jobId;
			location.replace(url);
		}
	}

	jobs.forceError = function()
	{
		var p = 'CAUTION!!\n';
		p += 'Use this for jobs that have crashed and can\'t be aborted normally.\n';
		var msg = prompt(p, 'Job has crashed');
		if (msg == null) return;
		
		var jobId = Data.get('page-data', 'item-id');
		var url = 'index.jsp?ID='+App.getSessionId();
		url += '&cmd=ForceError&item_id='+jobId;
		url += '&msg='+encodeURIComponent(msg);
		location.replace(url);
	}

	
	jobs.restartJob = function(event, options)
	{
		var parameterVersion = Data.int('page-data', 'job-parameter-version');
		var latestVersion = Data.int('page-data', 'latest-parameter-version');
		var breakPoint = Data.get('page-data', 'breakpoint');
		var jobId = Data.get('page-data', 'item-id');
		var clearDryRun = event ? Data.int(event.currentTarget, 'clear-dry-run', 0) : 0;
		
		var useLatestConfiguration = 0;
		var resumeFromBreakPoint = 0;
		if (parameterVersion != latestVersion || breakPoint)
		{
			if (!options)
			{
				var url = 'restart_job_options.jsp?';
				Dialogs.openPopup(url, 'RestartJobOptions', 500, 300);
				return;
			}
			else
			{
				useLatestConfiguration = options.useLatestConfiguration || 0;
				resumeFromBreakPoint = options.resumeFromBreakPoint || 0;
			}
		}
		var url = 'index.jsp?ID='+App.getSessionId();
		url += '&cmd=RestartJob&item_id='+jobId;
		url += '&useLatestConfiguration='+useLatestConfiguration;
		url += '&resumeFromBreakPoint='+resumeFromBreakPoint;
		if (clearDryRun) url += '&clearDryRun=' + clearDryRun;
		location.replace(url);
	}

	jobs.reconfigureJob = function()
	{
		var jobId = Data.get('page-data', 'item-id');
		var url = '../../common/plugin/index.jsp?ID='+App.getSessionId();
		url += '&cmd=ConfigureJob&job_id='+jobId;
		location.replace(url);
	}

	jobs.sendProgressUpdateRequest = function()
	{
		var jobId = Data.get('page-data', 'item-id');
		var url = 'ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=GetProgress&item_id='+jobId+'&'+Math.random();
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, jobs.progressUpdateCallback);
		request.send(null);
	}
	
	jobs.progressUpdateCallback = function(request)
	{
		var progress = JSON.parse(request.responseText);
		var nextAction = jobs.sendProgressUpdateRequest;
		var nextTimeout = 1500;
		if (progress && progress.status == 'ok')
		{
			var status = progress.jobStatus;
			var message = progress.message;
			var percentComplete = progress.percentComplete;
			var runningTime = progress.runningTime;
			
			if (status != Data.get('page-data', 'job-status')) 
			{
				nextAction = App.reloadWindow;
				nextTimeout = 100;
			}
			if (status == 'EXECUTING' || status == 'ABORTING')
			{
				jobs.displayProgress(percentComplete, message, runningTime);
			}
		}
		setTimeout(nextAction, nextTimeout);
	}

	jobs.displayProgress = function(percentDone, message, runningTime)
	{
		// Percent complete
		var doneElement = Doc.element('percentDone');
		var remainElement = Doc.element('percentRemain');
		var unknownElement = Doc.element('percentUnknown');
		if (percentDone == -1)
		{
			Doc.hide('percentDone');
			Doc.hide('percentRemain');
			Doc.show('percentUnknown');
			Doc.hide('percentText');
		}
		else
		{
			Doc.hide('percentUnknown');
			Doc.show('percentText');
			Doc.element('percentText').innerHTML = percentDone+'%';
			doneElement.style.width = percentDone+'%';
			remainElement.style.width = (100-percentDone)+'%';
			Doc.showHide('percentDone', percentDone > 0);
			Doc.showHide('percentRemain', percentDone < 100);
		}

		// Message
		Doc.element('message').innerHTML = message;

		// Running time
		Doc.element('runningTime').innerHTML = runningTime;
	}
	
	return jobs;
}();

Doc.onLoad(Jobs.initPage);
